import getLog from './get-log';
// eslint-disable-next-line no-new-func
const idempotentFunction = Function('obj', 'return obj');

const getSecretsManager = pool => {
  return {
    deleteSecret: pool.deleteSecret,
    getSecret: pool.getSecret,
    setSecret: pool.setSecret
  };
};

const createAdapter = ({
  maybeThrowResourceError,
  wrapMethod,
  wrapEventFilter,
  wrapDispose,
  validateEventFilter,
  loadEvents,
  importEventsStream,
  exportEventsStream,
  incrementalImport,
  getNextCursor,
  importSecretsStream,
  exportSecretsStream,
  init,
  drop
}, {
  connect,
  loadEventsByCursor,
  loadEventsByTimestamp,
  ensureEventSubscriber,
  removeEventSubscriber,
  getEventSubscribers,
  getLatestEvent,
  saveEvent,
  initEvents,
  initSecrets,
  initFinal,
  dropEvents,
  dropSecrets,
  dropFinal,
  dispose,
  injectEvent,
  freeze,
  unfreeze,
  shapeEvent,
  loadSnapshot,
  saveSnapshot,
  dropSnapshot,
  beginIncrementalImport,
  commitIncrementalImport,
  rollbackIncrementalImport,
  pushIncrementalImport,
  getSecret,
  setSecret,
  deleteSecret,
  loadSecrets,
  injectSecret
}, connectionDependencies, options) => {
  const log = getLog(`createAdapter`);
  const config = { ...options
  };
  let bucketSize = 100;
  const {
    snapshotBucketSize
  } = config;

  if (snapshotBucketSize !== undefined && Number.isSafeInteger(snapshotBucketSize) && snapshotBucketSize > 0) {
    bucketSize = snapshotBucketSize;
    log.debug(`snapshot bucket size explicitly set to ${bucketSize}`);
  } else {
    log.debug(`snapshot bucket size defaulted to ${bucketSize}`);
  }

  const primalProps = {
    disposed: false,
    validateEventFilter,
    isInitialized: false,
    maybeThrowResourceError,
    bucketSize,
    getNextCursor: getNextCursor.bind(null),
    counters: new Map()
  };
  const emptyProps = {};
  const adapterPool = { ...primalProps,
    ...emptyProps
  };
  adapterPool.connectPromise = new Promise(resolve => {
    adapterPool.connectPromiseResolve = resolve.bind(null, null);
  }).then(connect.bind(null, adapterPool, connectionDependencies, config));
  const connectedProps = {
    injectEvent: wrapMethod(adapterPool, injectEvent),
    injectSecret: wrapMethod(adapterPool, injectSecret),
    loadEventsByCursor: wrapMethod(adapterPool, loadEventsByCursor),
    loadEventsByTimestamp: wrapMethod(adapterPool, loadEventsByTimestamp),
    deleteSecret: wrapMethod(adapterPool, deleteSecret),
    getSecret: wrapMethod(adapterPool, getSecret),
    setSecret: wrapMethod(adapterPool, setSecret),
    initEvents: wrapMethod(adapterPool, initEvents),
    initSecrets: wrapMethod(adapterPool, initSecrets),
    initFinal: wrapMethod(adapterPool, initFinal),
    dropEvents: wrapMethod(adapterPool, dropEvents),
    dropSecrets: wrapMethod(adapterPool, dropSecrets),
    dropFinal: wrapMethod(adapterPool, dropFinal),
    waitConnect: wrapMethod(adapterPool, idempotentFunction),
    shapeEvent
  };
  Object.assign(adapterPool, connectedProps);
  const adapter = {
    loadEvents: wrapMethod(adapterPool, wrapEventFilter(loadEvents)),
    importEvents: importEventsStream.bind(null, adapterPool),
    exportEvents: exportEventsStream.bind(null, adapterPool),
    getLatestEvent: wrapMethod(adapterPool, getLatestEvent),
    saveEvent: wrapMethod(adapterPool, saveEvent),
    init: wrapMethod(adapterPool, init),
    drop: wrapMethod(adapterPool, drop),
    dispose: wrapDispose(adapterPool, dispose),
    freeze: wrapMethod(adapterPool, freeze),
    unfreeze: wrapMethod(adapterPool, unfreeze),
    getNextCursor: getNextCursor.bind(null),
    getSecretsManager: wrapMethod(adapterPool, getSecretsManager),
    loadSnapshot: wrapMethod(adapterPool, loadSnapshot),
    saveSnapshot: wrapMethod(adapterPool, saveSnapshot),
    dropSnapshot: wrapMethod(adapterPool, dropSnapshot),
    pushIncrementalImport: wrapMethod(adapterPool, pushIncrementalImport),
    beginIncrementalImport: wrapMethod(adapterPool, beginIncrementalImport),
    commitIncrementalImport: wrapMethod(adapterPool, commitIncrementalImport),
    rollbackIncrementalImport: wrapMethod(adapterPool, rollbackIncrementalImport),
    incrementalImport: wrapMethod(adapterPool, incrementalImport),
    loadSecrets: wrapMethod(adapterPool, loadSecrets),
    importSecrets: importSecretsStream.bind(null, adapterPool),
    exportSecrets: exportSecretsStream.bind(null, adapterPool),
    ensureEventSubscriber: wrapMethod(adapterPool, ensureEventSubscriber),
    removeEventSubscriber: wrapMethod(adapterPool, removeEventSubscriber),
    getEventSubscribers: wrapMethod(adapterPool, getEventSubscribers)
  };
  Object.assign(adapterPool, adapter);
  return Object.freeze(adapter);
};

export default createAdapter;
//# sourceMappingURL=create-adapter.js.map