function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume(key === "return" ? "return" : "next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _AwaitValue(value) { this.wrapped = value; }

import { Readable } from 'stream';
import { BATCH_SIZE, MAINTENANCE_MODE_AUTO } from './constants';
import { AlreadyFrozenError, AlreadyUnfrozenError } from './frozen-errors';

async function startProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.freeze();
    } catch (error) {
      if (!AlreadyFrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function endProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.unfreeze();
    } catch (error) {
      if (!AlreadyUnfrozenError.is(error)) {
        throw error;
      }
    }
  }
}

function generator(_x) {
  return _generator.apply(this, arguments);
}

function _generator() {
  _generator = _wrapAsyncGenerator(function* (context) {
    const {
      pool
    } = context;
    yield _awaitAsyncGenerator(pool.waitConnect());
    yield _awaitAsyncGenerator(startProcessSecrets(context));

    while (true) {
      const {
        secrets,
        idx
      } = yield _awaitAsyncGenerator(pool.loadSecrets({
        idx: context.idx,
        limit: BATCH_SIZE
      }));

      for (const secret of secrets) {
        const chunk = Buffer.from(JSON.stringify(secret) + '\n', 'utf8');
        yield chunk;
        context.idx = idx;
      }

      if (secrets.length === 0) {
        yield _awaitAsyncGenerator(endProcessSecrets(context));
        return;
      }
    }
  });
  return _generator.apply(this, arguments);
}

const exportSecretsStream = (pool, {
  idx = null,
  maintenanceMode = MAINTENANCE_MODE_AUTO
} = {}) => {
  if (pool.loadSecrets === undefined) throw new Error('loadSecrets is not defined for this adapter');
  const context = {
    pool,
    idx,
    maintenanceMode
  };
  const stream = Readable.from(generator(context));
  Object.defineProperty(stream, 'idx', {
    get() {
      return context.idx;
    }

  });
  return stream;
};

export default exportSecretsStream;
//# sourceMappingURL=export-secrets.js.map