export const ResourceAlreadyExistError = function (message) {
  Error.call(this);
  this.code = 406;
  this.message = message;
  Object.assign(this, {
    name: 'ResourceAlreadyExistError'
  });

  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, ResourceAlreadyExistError);
  } else {
    this.stack = new Error().stack;
  }
};
void (ResourceAlreadyExistError.is = error => error != null && error.name === 'ResourceAlreadyExistError');
export const ResourceNotExistError = function (message) {
  Error.call(this);
  this.code = 410;
  this.message = message;
  Object.assign(this, {
    name: 'ResourceNotExistError'
  });

  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, ResourceNotExistError);
  } else {
    this.stack = new Error().stack;
  }
};
void (ResourceNotExistError.is = error => error != null && error.name === 'ResourceNotExistError');
ResourceAlreadyExistError.prototype = Object.create(Error.prototype, {
  constructor: {
    enumerable: true,
    value: ResourceAlreadyExistError
  }
});
ResourceNotExistError.prototype = Object.create(Error.prototype, {
  constructor: {
    enumerable: true,
    value: ResourceNotExistError
  }
});
const allowedResourceErrorsConstructors = Array.from([ResourceAlreadyExistError, ResourceNotExistError]);
export const maybeThrowResourceError = errors => {
  if (!Array.isArray(errors) || !errors.reduce((acc, error) => acc && error instanceof Error, true)) {
    throw new Error(`Resource errors array is invalid ${errors}`);
  }

  if (errors.length === 0) {
    return;
  }

  const messages = errors.map(error => error.message).join('\n');
  const stacks = errors.map(error => error.stack).join('\n');
  const errorConstructors = errors.reduce((acc, error) => acc.add(error.constructor), new Set());
  let ErrorConstructor = errorConstructors.size === 1 ? [...errorConstructors.values()][0] : Error;

  if (!allowedResourceErrorsConstructors.includes(ErrorConstructor)) {
    ErrorConstructor = Error;
  }

  const error = new ErrorConstructor(messages);
  error.stack = stacks;
  throw error;
};
//# sourceMappingURL=resource-errors.js.map