"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validate = validate;
exports.isTimestampFilter = isTimestampFilter;
exports.isCursorFilter = isCursorFilter;
exports.AdapterConfigSchema = exports.EventFilterSchema = void 0;

var t = _interopRequireWildcard(require("io-ts"));

var _These = require("fp-ts/These");

var _Either = require("fp-ts/Either");

var _PathReporter = require("io-ts/lib/PathReporter");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function validate(schema, params, errorName) {
  const validationResult = schema.decode(params);

  if (!(0, _These.isRight)(validationResult)) {
    const messages = _PathReporter.PathReporter.report(validationResult);

    const error = new Error(messages.join('\r\n'));

    if (errorName != null) {
      error.name = errorName;
    }

    throw error;
  }

  return params;
}
/* eslint-disable @typescript-eslint/no-unused-vars */


const EventFilterCommonSchema = t.intersection([t.type({
  limit: t.Int
}), t.partial({
  eventTypes: t.union([t.array(t.string), t.null]),
  aggregateIds: t.union([t.array(t.string), t.null]),
  eventsSizeLimit: t.Int
})]);
const TimestampFilterFieldsSchema = t.partial({
  startTime: t.Int,
  finishTime: t.Int
});
const TimestampFilterSchema = t.intersection([EventFilterCommonSchema, TimestampFilterFieldsSchema]);
const CursorFilterFieldsSchema = t.type({
  cursor: t.union([t.string, t.null])
});
const CursorFilterSchema = t.intersection([EventFilterCommonSchema, CursorFilterFieldsSchema]);
const EventFilterSchemaSimple = t.intersection([EventFilterCommonSchema, TimestampFilterFieldsSchema, t.partial(CursorFilterFieldsSchema.props)]);
const EventFilterSchema = new t.Type('EventFilterSchema', EventFilterSchemaSimple.is, (u, c) => _Either.either.chain(EventFilterSchemaSimple.validate(u, c), filter => {
  if (isCursorFilter(filter)) {
    if (isTimestampFilter(filter)) {
      return t.failure(u, c, 'startTime and finishTime are not allowed in cursor filter');
    }

    return t.success(filter);
  } else if (isTimestampFilter(filter)) {
    if (filter.startTime !== undefined && filter.finishTime !== undefined) {
      if (filter.startTime > filter.finishTime) {
        return t.failure(u, c, `Event filter start time cannot be later than finishTime`);
      }
    }

    return t.success(filter);
  } else {
    return t.failure(u, c, 'Cursor or at least one of startTime or finishTime should be defined');
  }
}), t.identity);
exports.EventFilterSchema = EventFilterSchema;

function isTimestampFilter(filter) {
  return filter.startTime != null || filter.finishTime != null;
}

function isCursorFilter(filter) {
  return filter.cursor !== undefined;
} // eslint-disable-next-line @typescript-eslint/no-unused-vars


const AdapterConfigSchema = t.partial({
  snapshotBucketSize: t.Int
});
exports.AdapterConfigSchema = AdapterConfigSchema;
//# sourceMappingURL=types.js.map